=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-04-13 08:57:24 +0000
+++ bzrlib/builtins.py	2006-04-13 22:37:15 +0000
@@ -52,11 +52,22 @@
         raise BzrCommandError("%s is not in the same branch as %s" %
                              (e.path, file_list[0]))
 
+
+# XXX: Bad function name; should possibly also be a class method of
+# WorkingTree rather than a function.
 def internal_tree_files(file_list, default_branch=u'.'):
-    """\
-    Return a branch and list of branch-relative paths.
-    If supplied file_list is empty or None, the branch default will be used,
-    and returned file_list will match the original.
+    """Convert command-line paths to a WorkingTree and relative paths.
+
+    This is typically used for command-line processors that take one or
+    more filenames, and infer the workingtree that contains them.
+
+    The filenames given are not required to exist.
+
+    :param file_list: Filenames to convert.  
+
+    :param default_branch: Fallback tree path to use if file_list is empty or None.
+
+    :return: workingtree, [relative_paths]
     """
     if file_list is None or len(file_list) == 0:
         return WorkingTree.open_containing(default_branch)[0], file_list
@@ -908,7 +919,18 @@
 
 
 class cmd_init_repository(Command):
-    """Create a shared repository to keep branches in."""
+    """Create a shared repository to hold branches.
+
+    New branches created under the repository directory will store their revisions
+    in the repository, not in the branch directory, if the branch format supports
+    shared storage.
+
+    example:    
+        bzr init-repo repo
+        bzr init --format=metadir repo/trunk
+        cd repo/trunk
+        (add files here)
+    """
     takes_args = ["location"] 
     takes_options = [Option('format', 
                             help='Use a specific format rather than the'
@@ -926,8 +948,10 @@
         from bzrlib.transport import get_transport
         if format is None:
             format = BzrDirMetaFormat1()
-        get_transport(location).mkdir('')
-        newdir = format.initialize(location)
+        transport = get_transport(location)
+        if not transport.has('.'):
+            transport.mkdir('')
+        newdir = format.initialize_on_transport(transport)
         repo = newdir.create_repository(shared=True)
         repo.set_make_working_trees(trees)
 

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2006-01-19 23:14:42 +0000
+++ bzrlib/delta.py	2006-04-13 05:37:24 +0000
@@ -1,4 +1,4 @@
-# -*- coding: UTF-8 -*-
+# Copyright (C) 2005, 2006 Canonical
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -157,7 +157,8 @@
 
     specific_files
         If true, only check for changes to specified names or
-        files within them.
+        files within them.  Any unversioned files given have no effect
+        (but this might change in the future).
     """
 
     old_tree.lock_read()
@@ -181,8 +182,10 @@
     delta = TreeDelta()
     mutter('start compare_trees')
 
-    # TODO: match for specific files can be rather smarter by finding
-    # the IDs of those files up front and then considering only that.
+    # TODO: Rather than iterating over the whole tree and then filtering, we
+    # could diff just the specified files (if any) and their subtrees.  
+    # Perhaps should take a list of file-ids instead?   Need to indicate any
+    # ids or names which were not found in the trees.
 
     for file_id in old_tree:
         if file_id in new_tree:

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2006-02-28 14:45:12 +0000
+++ bzrlib/diff.py	2006-04-13 05:51:35 +0000
@@ -1,4 +1,4 @@
-# -*- coding: UTF-8 -*-
+# Copyright (C) 2004, 2005, 2006 Canonical Ltd.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
 
 from bzrlib.delta import compare_trees
 from bzrlib.errors import BzrError
+import bzrlib.errors as errors
 from bzrlib.symbol_versioning import *
 from bzrlib.trace import mutter
 
@@ -235,7 +236,6 @@
     external_diff_options
         If set, use an external GNU diff and pass these options.
     """
-
     old_tree.lock_read()
     try:
         new_tree.lock_read()
@@ -264,6 +264,8 @@
     # TODO: Generation of pseudo-diffs for added/deleted files could
     # be usefully made into a much faster special case.
 
+    _raise_if_doubly_unversioned(specific_files, old_tree, new_tree)
+
     if external_diff_options:
         assert isinstance(external_diff_options, basestring)
         opts = external_diff_options.split()
@@ -272,7 +274,6 @@
     else:
         diff_file = internal_diff
     
-
     delta = compare_trees(old_tree, new_tree, want_unchanged=False,
                           specific_files=specific_files)
 
@@ -306,7 +307,19 @@
             _maybe_diff_file_or_symlink(old_label, path, old_tree, file_id,
                                         new_label, path, new_tree,
                                         True, kind, to_file, diff_file)
+
     return has_changes
+
+
+def _raise_if_doubly_unversioned(specific_files, old_tree, new_tree):
+    """Complain if paths are not versioned in either tree."""
+    if not specific_files:
+        return
+    old_unversioned = old_tree.filter_unversioned_files(specific_files)
+    new_unversioned = new_tree.filter_unversioned_files(specific_files)
+    unversioned = old_unversioned.intersection(new_unversioned)
+    if unversioned:
+        raise errors.PathsNotVersionedError(sorted(unversioned))
     
 
 def get_prop_change(meta_modified):

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-04-13 08:57:24 +0000
+++ bzrlib/errors.py	2006-04-13 22:37:15 +0000
@@ -267,6 +267,17 @@
         self.path = path
 
 
+class PathsNotVersionedError(BzrNewError):
+    # used when reporting several paths are not versioned
+    """Path(s) are not versioned: %(paths_as_string)s"""
+
+    def __init__(self, paths):
+        from bzrlib.osutils import quotefn
+        BzrNewError.__init__(self)
+        self.paths = paths
+        self.paths_as_string = ' '.join([quotefn(p) for p in paths])
+
+
 class BadFileKindError(BzrError):
     """Specified file is of a kind that cannot be added.
 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-04-12 08:40:15 +0000
+++ bzrlib/repository.py	2006-04-13 04:25:59 +0000
@@ -1330,23 +1330,14 @@
     def initialize(self, a_bzrdir, shared=False):
         """Create a knit format 1 repository.
 
+        :param a_bzrdir: bzrdir to contain the new repository; must already
+            be initialized.
         :param shared: If true the repository will be initialized as a shared
                        repository.
-        XXX NOTE that this current uses a Weave for testing and will become 
-            A Knit in due course.
         """
-        from bzrlib.weavefile import write_weave_v5
-        from bzrlib.weave import Weave
-
-        # Create an empty weave
-        sio = StringIO()
-        bzrlib.weavefile.write_weave_v5(Weave(), sio)
-        empty_weave = sio.getvalue()
-
         mutter('creating repository in %s.', a_bzrdir.transport.base)
-        dirs = ['revision-store', 'knits', 'control']
-        files = [('control/inventory.weave', StringIO(empty_weave)), 
-                 ]
+        dirs = ['revision-store', 'knits']
+        files = []
         utf8_files = [('format', self.get_format_string())]
         
         self._upload_blank_content(a_bzrdir, dirs, files, utf8_files, shared)

=== modified file 'bzrlib/tests/blackbox/test_diff.py'
--- bzrlib/tests/blackbox/test_diff.py	2006-02-28 14:45:12 +0000
+++ bzrlib/tests/blackbox/test_diff.py	2006-04-13 05:51:35 +0000
@@ -1,5 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
-# -*- coding: utf-8 -*-
+# Copyright (C) 2005, 2006 by Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,7 +26,8 @@
 
 
 class TestDiff(ExternalBase):
-    def example_branch(test):
+
+    def make_example_branch(test):
         # FIXME: copied from test_too_much -- share elsewhere?
         test.runbzr('init')
         file('hello', 'wt').write('foo')
@@ -38,7 +38,7 @@
         test.runbzr('commit -m setup goodbye')
 
     def test_diff(self):
-        self.example_branch()
+        self.make_example_branch()
         file('hello', 'wt').write('hello world!')
         self.runbzr('commit -m fixing hello')
         output = self.runbzr('diff -r 2..3', backtick=1, retcode=1)
@@ -50,6 +50,23 @@
         os.unlink('moo')
         self.runbzr('diff')
 
+    def test_diff_nonexistent(self):
+        # Get an error from a file that does not exist at all
+        # (Malone #3619)
+        self.make_example_branch()
+        out, err = self.runbzr('diff does-not-exist', retcode=3)
+        self.assertContainsRe(err, 'not versioned.*does-not-exist')
+
+    def test_diff_unversioned(self):
+        # Get an error when diffing a non-versioned file.
+        # (Malone #3619)
+        self.make_example_branch()
+        self.build_tree(['unversioned-file'])
+        out, err = self.runbzr('diff unversioned-file', retcode=3)
+        self.assertContainsRe(err, 'not versioned.*unversioned-file')
+
+    # TODO: What should diff say for a file deleted in working tree?
+
     def example_branches(self):
         self.build_tree(['branch1/', 'branch1/file'], line_endings='binary')
         self.capture('init branch1')
@@ -102,8 +119,8 @@
 
 class TestCheckoutDiff(TestDiff):
 
-    def example_branch(self):
-        super(TestCheckoutDiff, self).example_branch()
+    def make_example_branch(self):
+        super(TestCheckoutDiff, self).make_example_branch()
         self.runbzr('checkout . checkout')
         os.chdir('checkout')
 
@@ -120,29 +137,30 @@
         self.runbzr('checkout branch2 checkouts/branch2')
         os.chdir('checkouts')
 
+
 class TestDiffLabels(TestDiff):
 
     def test_diff_label_removed(self):
-        super(TestDiffLabels, self).example_branch()
+        super(TestDiffLabels, self).make_example_branch()
         self.runbzr('remove hello')
         diff = self.run_bzr_captured(['diff'], retcode=1)
         self.assertTrue("=== removed file 'a/hello'" in diff[0])
 
     def test_diff_label_added(self):
-        super(TestDiffLabels, self).example_branch()
+        super(TestDiffLabels, self).make_example_branch()
         file('barbar', 'wt').write('barbar')
         self.runbzr('add barbar')
         diff = self.run_bzr_captured(['diff'], retcode=1)
         self.assertTrue("=== added file 'b/barbar'" in diff[0])
 
     def test_diff_label_modified(self):
-        super(TestDiffLabels, self).example_branch()
+        super(TestDiffLabels, self).make_example_branch()
         file('hello', 'wt').write('barbar')
         diff = self.run_bzr_captured(['diff'], retcode=1)
         self.assertTrue("=== modified file 'a/hello'" in diff[0])
 
     def test_diff_label_renamed(self):
-        super(TestDiffLabels, self).example_branch()
+        super(TestDiffLabels, self).make_example_branch()
         self.runbzr('rename hello gruezi')
         diff = self.run_bzr_captured(['diff'], retcode=1)
         self.assertTrue("=== renamed file 'a/hello' => 'b/gruezi'" in diff[0])

=== modified file 'bzrlib/tests/blackbox/test_shared_repository.py'
--- bzrlib/tests/blackbox/test_shared_repository.py	2006-04-04 00:28:28 +0000
+++ bzrlib/tests/blackbox/test_shared_repository.py	2006-04-13 03:48:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2005, 2006 by Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 
 from bzrlib.tests import TestCaseInTempDir
 import bzrlib.bzrdir
+from bzrlib.bzrdir import BzrDir
 import bzrlib.errors as errors
 
 class TestSharedRepo(TestCaseInTempDir):
@@ -33,6 +34,15 @@
         self.assertRaises(errors.NotBranchError, dir.open_branch)
         self.assertRaises(errors.NoWorkingTree, dir.open_workingtree)        
 
+    def test_init_repo_existing_dir(self):
+        """Make repo in existing directory.
+        
+        (Malone #38331)
+        """
+        out, err = self.run_bzr("init-repository", ".")
+        dir = BzrDir.open('.')
+        self.assertTrue(dir.open_repository())
+
     def test_init(self):
         self.run_bzr("init-repo", "a")
         self.run_bzr("init", "--format=metadir", "a/b")

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2006-03-02 06:23:15 +0000
+++ bzrlib/tree.py	2006-04-13 05:51:35 +0000
@@ -110,6 +110,17 @@
 
     def unlock(self):
         pass
+
+    def filter_unversioned_files(self, paths):
+        """Filter out paths that are not versioned.
+
+        :return: set of paths.
+        """
+        # NB: we specifically *don't* call self.has_filename, because for
+        # WorkingTrees that can indicate files that exist on disk but that 
+        # are not versioned.
+        pred = self.inventory.has_filename
+        return set((p for p in paths if not pred(p)))
         
         
 class RevisionTree(Tree):

